#! /usr/bin/env python
# -*- coding: UTF8 -*-

import os
import re
import mailbox
import sys
import email
import time
import logging

from creole.parsedico import parse_dico
from pyeole.process import system_out

dico = parse_dico()

# Entêtes de la liste des utilisateurs sans store zarafa
ORPHAN_LIST_HEADER = "Users without stores:"
ORPHAN_COLUMN_NAME = ".*(Username)"

# Commandes Zarafa
ZRF_ADMIN_CMD   = "zarafa-admin"
ZRF_DAGENT_CMD  = "zarafa-dagent"

# Répertoires
MAILDIR_DIRECTORY = "/home/mail"

# Fichiers
LOG_FILE = "/tmp/migration-vers-zarafa-" + time.strftime("%d%m%Y-%H%M.log")
logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG)

#
# Nom: listTopDir
# But: Liste les sous répertoires d'un répertoire (non récursif)
# Paramètres: Le répertoire à lister
#
def listTopDir(directory):
    return [d for d in os.listdir(directory) if os.path.isdir(os.path.join(directory, d))]

#
# Nom: getOrphansAccounts
# But: Liste les utilisateurs sans store zarafa
# Paramètres: aucun
#
def getOrphansAccounts():
    cmd = [ZRF_ADMIN_CMD,"--list-orphans"]
    ret = system_out(cmd, container="mail")
    orphans = ret[1]
    orphans = orphans.split("\n")
    userlist = []
    in_listing = False
    for elm in orphans:
        if re.match(".*---*",elm):
            continue

        if elm == ORPHAN_LIST_HEADER:
            in_listing = True
            continue

        if re.match(ORPHAN_COLUMN_NAME, elm) and in_listing:
            continue

        if in_listing:
            user = elm.replace('\t','')
            if user:
                userlist.append(user)

    return userlist

#
# Nom: fixOrphans
# But: Créer les store pour tous les utilisateurs listés par getOrphansAccounts
# Paramètres: aucun
#
def fixOrphans():
    orphans =  getOrphansAccounts()
    status = True
    for user in orphans:
        print("Working on user %s" % user)
        logging.info("User %s is orphan:\t\t Creating Zarafa Store" % user)
        cmd = [ZRF_ADMIN_CMD, "--create-store", user]
        ret = system_out(cmd, container="mail")
        if ret[0] != 0:
            logging.error("Creation of %s store failed" % user)
            status = False
        else:
            logging.info("Store of user %s created" % user)

    return(status)

#
# Nom: zarafaDeliver#
# But: Envois un mail via zarafa-dagent
# Paramètres: le mail
#
def zarafaDeliver(msg):
    cmd = [ZRF_DAGENT_CMD, "-s"]
    ret = system_out(cmd, container=mail, stdin=msg)
    if ret[0] != 0:
        logging.error("Error delivering mail :\n %s \n\n" % msg)
        return False
    else:
        return True

#
# Nom: migrateMaildir
# But: Envois tous les mails du maildir à zarafa-dagent
# Paramètres: le répertoire Maildir
#
def migrateMaildir(directory):
    maildirs = listTopDir(directory)
    err = 0
    for elm in maildirs:
        print("Working on Maildir %s" % elm)
        mdir = mailbox.Maildir(elm, email.message_from_file)
        for msg in mdir:
            mail = email.message_from_string(str(msg))
            if zarafaDeliver(mail):
                err = err + 1
    if err != 0:
        return False
    else:
        return True

if dico["activer_groupware"] != "oui":
    print("Zarafa n'est pas activé, aucun traitement")
else:
    fixed = fixOrphans()
    if fixed:
        print("All orphans are consolidated")
    else:
        print("Error on orphans consolidation")

    migrated = migrateMaildir(MAILDIR_DIRECTORY)
    if migrated:
        print("All mail migrated without error")
    else:
        print("Erros on mail migration")

